package com.cenqua.fisheye.user.plugin;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

/**
 * An authenticator that accepts requests coming from a list (comma seperated)
 * of preconfigured remote addresses. It also logs all calls to stdout.
 * This can be used to learn more about the lifecycle of
 * Authenticators in FishEye.
 * 
 * An example configuration in the config.xml is:
 *   <pre>
 *     &lt;custom
 *      classname="com.cenqua.fisheye.user.plugin.ExampleFishEyeHttpAuthenticator"
 *      auto-add="true"
 *      positive-cache-ttl="5mins"&gt;
 *      &lt;properties&gt;
 *        allow.from=127.0.0.1,192.168.0.1
 *      &lt;/properties&gt;
 *    &lt;/custom&gt;
 *    </pre>
 */
public class ExampleFishEyeHttpAuthenticator extends AbstractFishEyeAuthenticator {

    private Set remoteAddrs;

    public AuthToken checkRequest(HttpServletRequest request) {
        System.out.println("checkRequest(" + request.getRemoteAddr() + ")");

        String addr = request.getRemoteAddr();
        if (remoteAddrs.contains(addr)) {
            return new ExampleAuthToken(addr);
        }
        return null;
    }

    public boolean isRequestUserStillValid(String username, HttpServletRequest req) {
        return true;
    }

    public void init(Properties cfg) throws Exception {
        System.out.println("init(" + cfg + ")");
        String property = cfg.getProperty("allow.from");
        remoteAddrs = property != null ?
                new HashSet(Arrays.asList(property.split(","))) : new HashSet();
    }

    public void close() {
        System.out.println("close()");
        remoteAddrs = null;
    }

    public AuthToken checkPassword(String username, String password) {
        System.out.println("checkPassword(" + username + ", " + password + ")");
        return null;
    }

    public AuthToken recreateAuth(String username) {
        System.out.println("recreateAuth(" + username + ")");
        return null;
    }

    public boolean hasPermissionToAccess(AuthToken tok, String repname, String constraint) {
        System.out.println("hasPermissionToAccess(" + tok + ", " + repname + ", " + constraint + ")");
        return true;
    }

    private static class ExampleAuthToken implements AuthToken {
        private final String name;

        public ExampleAuthToken(String name) {
            this.name = name;
        }

        public String getUsername() {
            System.out.println("ExampleAuthToken.getUsername()");
            return name;
        }
        public String getEmail() {
            System.out.println("ExampleAuthToken.getEmail()");
            return name + "@email";
        }

        public String getDisplayName() {
            // just fudge a display name, capitalise the first letter of name
            System.out.println("ExampleAuthToken.getDisplayName()");
            return name.substring(0, 1).toUpperCase() + name.substring(1);
        }
    }
}
