package com.cenqua.fisheye.user.plugin;

import javax.servlet.http.HttpServletRequest;
import java.util.Properties;


/**
 * The authentication/authorisation plugin interface for FishEye.
 *
 * <b>NB: It is recommended to extend {@link AbstractFishEyeAuthenticator} instead
 * of implementing this interface directly as it is subject to change
 * in future versions.</b>
 *
 * <p>Classes implementing this interface must supply a public no-arg constructor.</p>
 *
 * <p>General lifecycle:</p>
 * <ul>
 *  <li>An instance is created by calling {@link Class#newInstance}
 *  <li>{@link #init} is called. If it throws an exception, then no further methods
 *      will be called on this instance.
 *  <li>The auth methods are called any number of times:
 *      {@link #checkPassword},
 *      {@link #recreateAuth},
 *      {@link #hasPermissionToAccess},
 *      {@link #checkPassword},
 *      {@link #checkRequest} and
 *      {@link #isRequestUserStillValid}.
 *      These methods may be called concurrently (they must be multi-thread safe)..
 *  <li>{@link #close} is called (or the JVM exits abruptly).
 * </ul>
 *
 */
public interface FishEyeAuthenticator {

    /**
     * Called to configure this authenticator.
     *
     * @param cfg the configuration properties, as specified
     *   in the &lt;properties&gt; element in the &lt;custom&gt; config.
     *   (non-null)
     * @throws Exception an error occurred during configuration.
     */
    void init(Properties cfg) throws Exception;

    /**
     * Called when this authenticator will no longer be used.
     */
    void close();

    /**
     * Called to check a user's password.
     *
     * <p>You should not make any assumptions about the arguments. For example,
     * they could be the empty string, or null.</p>
     *
     * @param username given username (may be null)
     * @param password given password (may be null)
     * @return an authentication token on successful authentication, or null otherwise.
     */
    AuthToken checkPassword(String username, String password);

    /**
     * Used to re-create a token for a user that was previously authenticated.
     * For example, this method is called when a user returns and they have
     * selected "remember me" when they logged in.
     *
     * @return a re-created token, or null if the user cannot be re-created
     *  without re-authentication
     */
    AuthToken recreateAuth(String username);

    /**
     * Check if a user has permission to access the given repository.
     *
     * <p>This method is <b>not</b>called when users are not logged in (anonymous users).
     * </p>
     *
     * <p>
     * If a custom constraint has been specified in config.xml, then that value is passed as
     * <code>constraint</code>. This is taken from the &lt;security&gt;&lt;custom constraint=""&gt;
     * setting in either &lt;repository&gt; or &lt;repository-defaults&gt;.
     * </p>
     * <p>
     * A constraint could be used (for example) to specify a group that a user must belong to in
     * order to access a repository.
     * </p>
     *
     * @param tok an authentication token as returned by {@link #checkPassword}
     *      or {@link #recreateAuth} (not null)
     * @param repname the symbolic name of the repository (not null)
     * @param constraint a constraint as specified in config.xml (or null if not specified).
     */
    boolean hasPermissionToAccess(AuthToken tok, String repname, String constraint);

    /**
     * Called to allow implementations to determine if the request is
     * pre-authenticated, bypassing FishEye's own HTTP authentication.
     *
     * @param request the servlet request to check
     * @return an authentication token if the request is authenticated,
     * or <code>null</code> otherwise.
     */
    AuthToken checkRequest(HttpServletRequest request);

    /**
     * Checks that the given username is still valid for the request.
     *
     * @param username the UserName, must not be <code>null</code>
     * @param req the request to check the username against
     * @return <code>true</code> if the username is still valid,
     * <code>false</code> otherwise.
     */
    boolean isRequestUserStillValid(String username, HttpServletRequest req);
}
